/**
 * ==============================================
 *  DQL : Descartes Query Language
 * ==============================================
 *
 * (c) Copyright 2014-2016, by Juergen Walter, Fabian Gorsler, Fabian Brosig and other contributors.
 *
 * Project Info:   http://descartes.tools/dql
 *
 * All rights reserved. This software is made available under the terms of the
 * Eclipse Public License (EPL) v1.0 as published by the Eclipse Foundation
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * This software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the Eclipse Public License (EPL)
 * for more details.
 *
 * You should have received a copy of the Eclipse Public License (EPL)
 * along with this software; if not visit http://www.eclipse.org or write to
 * Eclipse Foundation, Inc., 308 SW First Avenue, Suite 110, Portland, 97204 USA
 * Email: license (at) eclipse.org
 *
 * [Java is a trademark or registered trademark of Sun Microsystems, Inc.
 * in the United States and other countries.]
 */
package tools.descartes.pavo.types;

import java.awt.Color;
import java.math.BigDecimal;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.StandardChartTheme;
import org.jfree.chart.axis.CategoryAxis;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.axis.ValueAxis;
import org.jfree.chart.plot.CategoryPlot;
import org.jfree.chart.plot.Plot;
import org.jfree.chart.plot.ValueMarker;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.category.CategoryItemRenderer;
import org.jfree.chart.renderer.category.StatisticalBarRenderer;
import org.jfree.chart.renderer.xy.ClusteredXYBarRenderer;
import org.jfree.chart.renderer.xy.XYBarRenderer;
import org.jfree.data.statistics.DefaultStatisticalCategoryDataset;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;
import org.jfree.ui.RectangleAnchor;
import org.jfree.ui.TextAnchor;

import tools.descartes.pavo.controller.VisualizationController;
import tools.descartes.pavo.result.AbstractQuantitativeResult;
import tools.descartes.pavo.result.ResultContainerPavo;
import tools.descartes.pavo.result.SeriesResultPavo;
import tools.descartes.pavo.result.ValueResultPavo;
import tools.descartes.pavo.util.Point;
import tools.descartes.pavo.view.View;

/**
 * Class to visualise the current result as BarChart if possible. Can visualise
 * results constiting out of value results or series results (Values are shown
 * as horizontal markers). Does not do any configurations for the control
 * buttons
 * 
 * @author Maximilian König <max.c.koenig@t-online.de>
 * @since 2016
 */
public class BarChart extends AbstractVisualizationType implements IVisualizationType {

	double barwidth = 0.2;
	JSpinner mySpinner;

	public BarChart(ResultContainerPavo resultN) {
		super(resultN);
	}

	public JFreeChart visualizeConfidence() {

		final DefaultStatisticalCategoryDataset dataset = new DefaultStatisticalCategoryDataset();

		CategoryAxis xAxis = new CategoryAxis(this.getXDescription());
		final ValueAxis yAxis = new NumberAxis(this.getYDescription());
		final CategoryItemRenderer renderer = new StatisticalBarRenderer();
		final CategoryPlot plot = new CategoryPlot(dataset, xAxis, yAxis, renderer);
		JFreeChart chart = new JFreeChart("", JFreeChart.DEFAULT_TITLE_FONT, plot, true);

		for (AbstractQuantitativeResult r : this.getResult().getResults()) {
			if (r instanceof SeriesResultPavo) {

				for (Point p : ((SeriesResultPavo) r).getValues()) {
					BigDecimal conf;
					if (p.getConf() == null)
						conf = BigDecimal.valueOf(0.0);
					else
						conf = p.getConf().getIntervalHalfLength();

					dataset.add(p.getY(), conf, ((SeriesResultPavo) r).getSeriesName(), p.getX());
				}
			}
		}
		plot.setBackgroundPaint(Color.white);
		plot.setRangeGridlinePaint(Color.lightGray);
		plot.setDomainGridlinePaint(Color.lightGray);
		return chart;
	}

	@Override
	public JFreeChart visualize() {
		ChartFactory.setChartTheme(StandardChartTheme.createLegacyTheme());
		XYSeriesCollection dataset = new XYSeriesCollection();
		JFreeChart chart = ChartFactory.createXYBarChart("", this.getXDescription(), false, this.getYDescription(),
				dataset);
		Boolean visualizeValuesAsLines = false;
		for (AbstractQuantitativeResult r : this.getResult().getResults()) {
			if (r instanceof SeriesResultPavo) {
				visualizeValuesAsLines = true;
			}
		}
		if (!visualizeValuesAsLines) {
			Plot plot = chart.getPlot();
			// TODO remove range
		}

		for (AbstractQuantitativeResult r : this.getResult().getResults()) {
			if (r instanceof ValueResultPavo) {
				if (visualizeValuesAsLines) {
					ValueMarker marker = new ValueMarker(((ValueResultPavo) r).getValue());
					marker.setPaint(Color.green);
					marker.setLabel(((ValueResultPavo) r).getName());
					marker.setLabelAnchor(RectangleAnchor.BOTTOM_RIGHT);
					marker.setLabelTextAnchor(TextAnchor.TOP_RIGHT);
					((XYPlot) chart.getPlot()).addRangeMarker(marker);
					ValueAxis yAxis = ((XYPlot) chart.getPlot()).getRangeAxis();
					if (marker.getValue() > yAxis.getUpperBound())
						yAxis.setRange(yAxis.getLowerBound(), marker.getValue() + 1);
				} else {
					String name = "";
					// TODO Generate minmal but unique identifier for result
					// for (dof dof : ((ValueResultPavo) r).getDofs()) {
					// name = name + " " + dof.getDofvalue();
					// // dof.getDofname()
					// }
					if (((ValueResultPavo) r).getDofs().size() == 1) {
						name = name + ((int) ((ValueResultPavo) r).getDofs().get(0).getDofvalue());
					} else {
						name = name + ((int) ((ValueResultPavo) r).getDofs().get(1).getDofvalue());
					}

					XYSeries bar = new XYSeries(name);
					bar.add(0, ((ValueResultPavo) r).getValue());
					dataset.addSeries(bar);
				}

			} else if (r instanceof SeriesResultPavo) {
				XYSeries series1 = new XYSeries(((SeriesResultPavo) r).getSeriesName());
				for (int j = 0; j < ((SeriesResultPavo) r).getValues().size(); j++) {
					series1.add(((SeriesResultPavo) r).getValues().get(j).getX(),
							((SeriesResultPavo) r).getValues().get(j).getY());
				}

				dataset.addSeries(series1);
			}
		}

		XYPlot plot = chart.getXYPlot();

		plot.setRenderer(new ClusteredXYBarRenderer(barwidth, true));
		XYBarRenderer.setDefaultShadowsVisible(false);
		return chart;
	}

	@Override
	public boolean canVisualize() {
		for (AbstractQuantitativeResult r : this.getResult().getResults()) {
			if (!(r instanceof ValueResultPavo) && !(r instanceof SeriesResultPavo)) {
				return false;
			}
		}
		return true;
	}

	public JPanel getEditViewComponent(VisualizationController c) {
		JPanel panel = new JPanel();
		JRadioButton a = View.createConfidence(c.getActionListenerConfidence());
		panel.add(a);

		try {
			if (c.getView().getConfidenceRadio().isSelected()) {
				a.setSelected(true);
			}
		} catch (NullPointerException e) {

		}

		c.getView().addConfidence(a);

		panel.add(View.createMinMaxMean(c.createActionListenerMin(), c.createActionListenerMax(),
				c.createActionListenerMean()));
		mySpinner = new JSpinner();
		mySpinner.setModel(new SpinnerNumberModel(1 - barwidth, 0.1, 2, 0.1));
		mySpinner.addChangeListener(new ChangeListener() {

			@Override
			public void stateChanged(ChangeEvent e) {
				barwidth = 1.0 - (double) mySpinner.getValue();
				c.updateChart(visualize());

			}
		});
		JPanel spinnerpanel = new JPanel();
		spinnerpanel.add(new JLabel("Bar Width:"));
		spinnerpanel.add(mySpinner);
		panel.add(spinnerpanel);

		return panel;

	}

}